@@ -1,18 +1,16 @@
 import copy
+import random
 import traceback
-from argparse import ArgumentError
 from itertools import chain
-from typing import Any, Dict, List, Type, Union
+from typing import Any, Dict, List, NoReturn, Type, Union
 
 from meme_generator.exception import (
     ArgMismatch,
-    ImageNumberMismatch,
     MemeGeneratorException,
-    TextNumberMismatch,
     TextOrNameNotEnough,
     TextOverLength,
 )
-from meme_generator.meme import Meme
+from meme_generator.meme import Meme, MemeParamsType
 from nonebot import on_command, on_message
 from nonebot.adapters.onebot.v11 import Bot as V11Bot
 from nonebot.adapters.onebot.v11 import GroupMessageEvent as V11GMEvent
@@ -31,15 +29,17 @@ from nonebot.adapters.onebot.v12 import Message as V12Msg
 from nonebot.adapters.onebot.v12 import MessageEvent as V12MEvent
 from nonebot.adapters.onebot.v12 import MessageSegment as V12MsgSeg
 from nonebot.adapters.onebot.v12.permission import PRIVATE
-from nonebot.exception import AdapterException
+from nonebot.exception import AdapterException, ParserExit
 from nonebot.log import logger
 from nonebot.matcher import Matcher
 from nonebot.params import CommandArg, Depends
 from nonebot.permission import SUPERUSER
 from nonebot.plugin import PluginMetadata
+from nonebot.rule import ArgumentParser, parser_message
 from nonebot.typing import T_Handler, T_State
 from pypinyin import Style, pinyin
 
+from .config import memes_config
 from .depends import (
     IMAGE_SOURCES_KEY,
     TEXTS_KEY,
@@ -66,7 +66,7 @@ __plugin_meta__ = PluginMetadata(
     extra={
         "unique_name": "memes",
         "author": "meetwq <meetwq@gmail.com>",
-        "version": "0.4.0",
+        "version": "0.4.1",
     },
 )
 
@@ -231,6 +231,54 @@ async def _(matcher: Matcher, msg: Union[V11Msg, V12Msg] = CommandArg()):
     await matcher.finish("\n".join(messages))
 
 
+async def process(
+    bot: Union[V11Bot, V12Bot],
+    matcher: Matcher,
+    meme: Meme,
+    image_sources: List[ImageSource],
+    texts: List[str],
+    users: List[User],
+    args: Dict[str, Any] = {},
+):
+    images: List[bytes] = []
+    user_infos: List[UserInfo] = []
+
+    try:
+        for image_source in image_sources:
+            images.append(await image_source.get_image())
+    except PlatformUnsupportError as e:
+        await matcher.finish(f"当前平台 “{e.platform}” 暂不支持获取头像，请使用图片输入")
+    except (NetworkError, AdapterException):
+        logger.warning(traceback.format_exc())
+        await matcher.finish("图片下载出错，请稍后再试")
+
+    try:
+        for user in users:
+            user_infos.append(await user.get_info())
+        args["user_infos"] = user_infos
+    except (NetworkError, AdapterException):
+        logger.warning("用户信息获取失败\n" + traceback.format_exc())
+
+    try:
+        result = await meme(images=images, texts=texts, args=args)
+    except TextOverLength as e:
+        await matcher.finish(f"文字 “{e.text}” 长度过长")
+    except ArgMismatch:
+        await matcher.finish("参数解析错误")
+    except TextOrNameNotEnough:
+        await matcher.finish("文字或名字数量不足")
+    except MemeGeneratorException:
+        logger.warning(traceback.format_exc())
+        await matcher.finish("出错了，请稍后再试")
+
+    if isinstance(bot, V11Bot):
+        await matcher.finish(V11MsgSeg.image(result))
+    else:
+        resp = await bot.upload_file(type="data", name="memes", data=result.getvalue())
+        file_id = resp["file_id"]
+        await matcher.finish(V12MsgSeg.image(file_id))
+
+
 def handler(meme: Meme) -> T_Handler:
     async def handle(
         bot: Union[V11Bot, V12Bot],
@@ -245,19 +293,25 @@ def handler(meme: Meme) -> T_Handler:
         users: List[User] = state[USERS_KEY]
         image_sources: List[ImageSource] = state[IMAGE_SOURCES_KEY]
 
-        images: List[bytes] = []
         texts: List[str] = []
         args: Dict[str, Any] = {}
-        user_infos: List[UserInfo] = []
+
+        async def finish(msg: str) -> NoReturn:
+            logger.info(msg)
+            if memes_config.memes_prompt_params_error:
+                matcher.stop_propagation()
+                await matcher.finish(msg)
+            await matcher.finish()
 
         if meme.params_type.args_type and (parser := meme.params_type.args_type.parser):
             parser = copy.deepcopy(parser)
             parser.add_argument("texts", nargs="*", default=[])
+            parser.__class__ = ArgumentParser
+            parser_message.set("")
             try:
                 parse_result = vars(parser.parse_args(raw_texts))
-            except (ArgumentError, SystemExit) as e:
-                logger.warning(f"参数解析错误: {e}")
-                return
+            except ParserExit as e:
+                await finish(f"参数解析错误:\n {e.message}")
             texts = parse_result["texts"]
             parse_result.pop("texts")
             args = parse_result
@@ -265,72 +319,30 @@ def handler(meme: Meme) -> T_Handler:
             texts = raw_texts
 
         if not (
-            (
-                meme.params_type.min_images
-                <= len(image_sources)
-                <= meme.params_type.max_images
-            )
-            and (meme.params_type.min_texts <= len(texts) <= meme.params_type.max_texts)
+            meme.params_type.min_images
+            <= len(image_sources)
+            <= meme.params_type.max_images
         ):
-            logger.info("输入 图片/文字 数量不符，跳过表情制作")
-            return
+            await finish(
+                f"输入图片数量不符，图片数量应为 {meme.params_type.min_images}"
+                + (
+                    f" ~ {meme.params_type.max_images}"
+                    if meme.params_type.max_images > meme.params_type.min_images
+                    else ""
+                )
+            )
+        if not (meme.params_type.min_texts <= len(texts) <= meme.params_type.max_texts):
+            await finish(
+                f"输入文字数量不符，文字数量应为 {meme.params_type.min_images}"
+                + (
+                    f" ~ {meme.params_type.max_images}"
+                    if meme.params_type.max_images > meme.params_type.min_images
+                    else ""
+                )
+            )
 
         matcher.stop_propagation()
-
-        try:
-            for image_source in image_sources:
-                images.append(await image_source.get_image())
-        except PlatformUnsupportError as e:
-            await matcher.finish(f"当前平台 “{e.platform}” 暂不支持获取头像，请使用图片输入")
-        except (NetworkError, AdapterException):
-            logger.warning(traceback.format_exc())
-            await matcher.finish("图片下载出错，请稍后再试")
-
-        try:
-            for user in users:
-                user_infos.append(await user.get_info())
-            args["user_infos"] = user_infos
-        except (NetworkError, AdapterException):
-            logger.warning("用户信息获取失败\n" + traceback.format_exc())
-
-        try:
-            result = await meme(images=images, texts=texts, args=args)
-        except ImageNumberMismatch:
-            await matcher.finish(
-                f"图片数量不符，图片数量应为 {meme.params_type.min_images}"
-                + (
-                    f" ~ {meme.params_type.max_images}"
-                    if meme.params_type.max_images > meme.params_type.min_images
-                    else ""
-                )
-            )
-        except TextNumberMismatch:
-            await matcher.finish(
-                f"文字数量不符，文字数量应为 {meme.params_type.min_images}"
-                + (
-                    f" ~ {meme.params_type.max_images}"
-                    if meme.params_type.max_images > meme.params_type.min_images
-                    else ""
-                )
-            )
-        except TextOverLength as e:
-            await matcher.finish(f"文字 “{e.text}” 长度过长")
-        except ArgMismatch:
-            await matcher.finish("参数解析错误")
-        except TextOrNameNotEnough:
-            await matcher.finish("文字或名字数量不足")
-        except MemeGeneratorException:
-            logger.warning(traceback.format_exc())
-            await matcher.finish("出错了，请稍后再试")
-
-        if isinstance(bot, V11Bot):
-            await matcher.finish(V11MsgSeg.image(result))
-        else:
-            resp = await bot.upload_file(
-                type="data", name="memes", data=result.getvalue()
-            )
-            file_id = resp["file_id"]
-            await matcher.finish(V12MsgSeg.image(file_id))
+        await process(bot, matcher, meme, image_sources, texts, users, args)
 
     return handle
 
@@ -357,5 +369,41 @@ def create_matchers():
             matcher.append_handler(handler(meme), parameterless=[split_msg_v11(meme)])
             matcher.append_handler(handler(meme), parameterless=[split_msg_v12(meme)])
 
+    async def random_handler(
+        bot: Union[V11Bot, V12Bot], state: T_State, matcher: Matcher
+    ):
+        texts: List[str] = state[TEXTS_KEY]
+        users: List[User] = state[USERS_KEY]
+        image_sources: List[ImageSource] = state[IMAGE_SOURCES_KEY]
+
+        random_meme = random.choice(
+            [
+                meme
+                for meme in meme_manager.memes
+                if (
+                    (
+                        meme.params_type.min_images
+                        <= len(image_sources)
+                        <= meme.params_type.max_images
+                    )
+                    and (
+                        meme.params_type.min_texts
+                        <= len(texts)
+                        <= meme.params_type.max_texts
+                    )
+                )
+            ]
+        )
+        await process(bot, matcher, random_meme, image_sources, texts, users)
+
+    random_matcher = on_message(command_rule(["随机表情"]), block=False, priority=12)
+    fake_meme = Meme("_fake", _, MemeParamsType())
+    random_matcher.append_handler(
+        random_handler, parameterless=[split_msg_v11(fake_meme)]
+    )
+    random_matcher.append_handler(
+        random_handler, parameterless=[split_msg_v12(fake_meme)]
+    )
+
 
 create_matchers()
