@@ -8,6 +8,8 @@ class Config(BaseModel, extra=Extra.ignore):
     memes_command_start: List[str] = []
     memes_disabled_list: List[str] = []
     memes_check_resources_on_startup: bool = True
+    memes_prompt_params_error: bool = False
+    memes_use_sender_when_no_image: bool = False
 
 
 memes_config = Config.parse_obj(get_driver().config.dict())
